/**
 *  \file ListenerSocket.cpp
 *  \brief The implementation of a socket to listen for incoming connections
 *         from clients.
 *
 *  Adapted from "Beej's Guide to Network Programming"
 *  http://beej.us/guide/bgnet/output/html/singlepage/bgnet.html
 *
 *  $LastChangedBy: jch231 $
 *  $Date: 2010-08-19 14:34:07 +1200 (Thu, 19 Aug 2010) $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/wait.h>

#include "ListenerSocket.h"
#include "Server.h"

#define BACKLOG 10	 ///< How many pending connections queue will hold


/**
 *  \brief Retrieve the IP address(es) of the given hostname 
 *  \param sa the address to lookup
 */
static void *get_in_addr(struct sockaddr *sa)
{
	if (sa->sa_family == AF_INET) {
		return &(((struct sockaddr_in*)sa)->sin_addr);
	}
  
	return &(((struct sockaddr_in6*)sa)->sin6_addr);
}


ListenerSocket::ListenerSocket(Server* server) {
  server_ = server;

	struct addrinfo hints, *servinfo, *p;
	int yes=1;
  int rv;

  // Set up the address hints structure with my IP address
	memset(&hints, 0, sizeof hints);
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_flags = AI_PASSIVE;
  
  // Lookup the hostname or IP address
	if ((rv = getaddrinfo(NULL, PORT, &hints, &servinfo)) != 0) {
		fprintf(stderr, "getaddrinfo: %s\n", gai_strerror(rv));
		exit(1);
	}
  
	// Loop through all the results and bind to the first one we can
	for(p = servinfo; p != NULL; p = p->ai_next) {
		if ((sock = socket(p->ai_family, p->ai_socktype, p->ai_protocol)) == -1) {
			perror("listener: socket");
			continue;
		}
    
    // Allow the socket to be reused if necessary
		if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) == -1) {
			perror("listener: setsockopt");
			exit(1);
		}
    
    // Bind to the given port
		if (bind(sock, p->ai_addr, p->ai_addrlen) == -1) {
			close(sock);
			perror("listener: bind");
			continue;
		}
    
		break;
	}
  
	if (p == NULL)  {
		fprintf(stderr, "listener: failed to bind\n");
		exit(2);
	}
  
  // We're done with this, so delete it
	freeaddrinfo(servinfo);
  
  // Listen for client connections
	if (listen(sock, BACKLOG) == -1) {
		perror("listen");
		exit(1);
	}
}

ListenerSocket::~ListenerSocket() { 
  close(sock);
}

void ListenerSocket::Run() {
  int client;
	struct sockaddr_storage their_addr; ///< The connector's address information
	socklen_t sin_size;
	char s[INET6_ADDRSTRLEN];

	while(!waiting) {
    printf("Waiting for a connection...\n");
    
    // Accept a connection from a client
		sin_size = sizeof their_addr;
		client = accept(sock, (struct sockaddr *)&their_addr, &sin_size);
		if (client == -1) {
			perror("accept");
			continue;
		}
    
    printf("Connected.\n");

		inet_ntop(their_addr.ss_family,
              get_in_addr((struct sockaddr *)&their_addr),
              s, sizeof s);
    
    server_->newConnection(client);
	}
}

