/**
 *  \file server/Outbox.h
 *  \brief Declares the Outbox class for the server
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 16:42:25 +1200 (Thu, 19 Aug 2010) $
 *
 */

#ifndef OUTBOX_H
#define OUTBOX_H

#include "TypeDef.h"
#include "ServerSocket.h"

/**
 *  \class Outbox
 *  \brief The Outbox build packets to send to the socket. One outbox is 
 *  associated with each socket connected to a client.
 */
class Outbox {
public:
  /**
   *  \brief Constructor.
   *  \param socket The socket that the outbox writes to.
   */
  Outbox(ServerSocket* socket);
  
  /**
   *  \brief Destructor.
   */
  ~Outbox(){}
  
  /**
   *  \brief Sedn a packet to the socket indicating that a login attempt failed.
   */
  void LoginFailed();
  
  /**
   *  \brief Send a packet to the socket indicating that a login attempt was
   *  successful.
   *  \param user The user id that successfully logged in.
   */
  void LoginSuccess(UserID user);
  
  /**
   *  \brief Send a packet to the socket to add a user.
   *  \param user The user to be added.
   */
  void AddUser(UserID user);
  
  /**
   *  \brief Send a packet to the socket to remove a user.
   *  \param user The user to remove.
   */
  void RemoveUser(UserID user);
  
  /**
   *  \brief Send a packet to the socket to start a chat.
   *  \param id The chat id of the chat to be started.
   */  
  void StartChat(ChatID id);

  /**
   *  \brief Send a message to a chat.
   *  \param message The message to send.
   *  \param chat The chat to send the message to.
   */
  void Send(Message message, ChatID chat);
  
  /**
   *  \brief Request the user id of the user successfully logged into this
   *  outbox.
   *  \returns The user id of the successfully logged in user, or "" if no
   *  user has successfully logged in.
   */
  UserID LoggedIn();
  
private:
  UserID user_;
  ServerSocket* connection_;
};

#endif /* OUTBOX_H */

